/*
 * Copyright (c) 2021 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef __ZONES_IMPL_H
#define __ZONES_IMPL_H

#include <jeffpc/types.h>
#include <jeffpc/array.h>
#include <jeffpc/cstr.h>
#include <jeffpc/error.h>

struct zone {
	const char *state;
	enum zone_predicate {
		ZP_UNKNOWN = 0, /* unknown/not implemented */
		ZP_CONST, /* always the same value */
	} pred;
	union {
		int constant; /* ZP_CONST */
	};
};

static inline int find_zone(const char *state, const struct zone *infos,
			    size_t ninfos)
{
	const struct zone *info;
	size_t i;

	for (i = 0; i < ninfos; i++) {
		info = &infos[i];

		if (strcasecmp(state, info->state))
			continue;

		switch (info->pred) {
			case ZP_UNKNOWN:
				return -1;
			case ZP_CONST:
				return info->constant;
		}

		panic("unknown zone predicate %d", info->pred);
	}

	return -1;
}

#endif
