/*
 * Copyright (c) 2021 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#ifndef __RPC_ANNOUNCE_PUB_H
#define __RPC_ANNOUNCE_PUB_H

#include <hlog-rpc/announce.h>

extern int announce_start(void);
extern void announce_stop(void);
extern void announce_publish(struct announce_msg *msg);

static inline void announce_publish_loc(enum announce_loc_msg_type type,
					uint8_t mode, const char *grid)
{
	struct announce_msg msg;

	announce_pack_loc_msg(&msg, type, mode, grid);
	announce_publish(&msg);
}

static inline void announce_publish_qso(enum announce_qso_msg_type type,
					const struct xuuid *uuid)
{
	struct announce_msg msg;

	announce_pack_qso_msg(&msg, type, uuid);
	announce_publish(&msg);
}

static inline void announce_publish_rig(enum announce_rig_msg_type type,
					uint64_t tx_freq, uint64_t rx_freq,
					uint32_t power)
{
	struct announce_msg msg;

	announce_pack_rig_msg(&msg, type, tx_freq, rx_freq, power);
	announce_publish(&msg);
}

#endif
