/*
 * Copyright (c) 2020-2021 Josef 'Jeff' Sipek <jeffpc@josefsipek.net>
 *
 * Permission is hereby granted, free of charge, to any person obtaining a copy
 * of this software and associated documentation files (the "Software"), to deal
 * in the Software without restriction, including without limitation the rights
 * to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
 * copies of the Software, and to permit persons to whom the Software is
 * furnished to do so, subject to the following conditions:
 *
 * The above copyright notice and this permission notice shall be included in
 * all copies or substantial portions of the Software.
 *
 * THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
 * IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
 * FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
 * AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
 * LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
 * OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN THE
 * SOFTWARE.
 */

#include "xlua_impl.h"
#include <hlog/index.h>

struct iterctx {
	struct index_iter_ctx ctx;
	struct qso *first_qso;
	bool first;
	bool call;
};

/* f(state, prev qso) -> next qso [-2,+1,e] */
static int __index_history_next(lua_State *L)
{
	struct iterctx *ctx;
	struct qso *qso;

	xlua_checkstack(L, 2, 2, 1);

	/*
	 * process argument
	 */
	ctx = lua_touserdata(L, -2);

	/*
	 * set up return values
	 */
	if (ctx->first)
		qso = ctx->first_qso;
	else if (ctx->call)
		qso = index_iter_call_history_next(&ctx->ctx);
	else
		qso = index_iter_history_next(&ctx->ctx);

	ctx->first_qso = NULL;
	ctx->first = false;

	xlua_pushqso_throws(L, qso_getref(qso), true);

	return 1;
}

/*
 * QSO iterator [-1..2,+2,e]
 *
 * f(asc) -> func, state
 * f(call, asc) -> func, state
 */
static int xlua_index_history(lua_State *L)
{
	struct iterctx *ctx;
	struct qso *first;
	const char *call;
	bool asc;

	xlua_checkstack(L, 1, 2, 2);

	/*
	 * process argument
	 */
	asc = lua_toboolean(L, -1);
	call = (lua_gettop(L) == 2) ? luaL_checkstring(L, -2) : NULL;

	/*
	 * set up return values:
	 *
	 * func, state
	 */
	lua_pushcfunction(L, __index_history_next);

	ctx = lua_newuserdata(L, sizeof(struct iterctx));

	if (!call) {
		first = index_iter_history_start(&ctx->ctx, asc);
	} else {
		struct str callstr = STR_STATIC_INITIALIZER(call);

		first = index_iter_call_history_start(&callstr, &ctx->ctx, asc);
	}

	ctx->first_qso = first;
	ctx->first = true;
	ctx->call = !!call;

	return 2;
}

static const luaL_Reg index_funcs[] = {
	{ "history", xlua_index_history },
	{ NULL, NULL },
};

void xlua_pushlib_index(lua_State *L)
{
	luaL_newlib(L, index_funcs);
}
